from __future__ import unicode_literals

import django
from django.conf.urls import url
from django.contrib import admin

from paypal.standard.ipn import views

if django.VERSION < (1, 7):
    import paypal.standard.ipn.admin  # noqa
    admin.autodiscover()


urlpatterns = [
    url(r'^ipn/$', views.ipn),
    url(r'^admin/', admin.site.urls),
]


if django.VERSION < (1, 8):
    from django.conf.urls import patterns
    urlpatterns = patterns('', *urlpatterns)



from __future__ import unicode_literals

from paypal.standard.signals import DeprecatedSignal

"""
These signals are different from IPN signals in that they are sent the second
the payment is failed or succeeds and come with the `item` object passed to
PayPalPro rather than an IPN object.

### SENDER is the item? is that right???

"""

# Sent when a recurring payments profile is created.
payment_profile_created = DeprecatedSignal("payment_profile_created is deprecated. Use the return value from "
                                           "createRecurringPaymentsProfile directly, or pass nvp_handler to PayPalPro")

# Sent when a payment is successfully processed.
payment_was_successful = DeprecatedSignal("payment_was_successful is deprecated. Use the return value from "
                                          "doDirectPayment, doExpressCheckoutPayment or pass nvp_handler to PayPalPro")

payment_was_flagged = DeprecatedSignal("payment_was_flagged is deprecated. It has never done anything useful")

recurring_cancel = DeprecatedSignal("recurring_cancel is deprecated. Use the return value from "
                                    "manangeRecurringPaymentsProfileStatus directly")

recurring_suspend = DeprecatedSignal("recurring_suspend is deprecated. Use the return value from "
                                     "manangeRecurringPaymentsProfileStatus directly")

recurring_reactivate = DeprecatedSignal("recurring_reactivate is deprecated. Use the return value from "
                                        "manangeRecurringPaymentsProfileStatus directly")



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

try:
    from django.contrib.auth import get_user_model
except ImportError:
    from django.contrib.auth.models import User
else:
    User = get_user_model()
# With the default User model these will be 'auth.User' and 'auth.user'
# so instead of using orm['auth.User'] we can use orm[user_orm_label]
user_orm_label = '%s.%s' % (User._meta.app_label, User._meta.object_name)
user_model_label = '%s.%s' % (User._meta.app_label, User._meta.module_name)

class Migration(SchemaMigration):

    def forwards(self, orm):

        # Changing field 'PayPalNVP.ipaddress'
        db.alter_column('paypal_nvp', 'ipaddress', self.gf('django.db.models.fields.GenericIPAddressField')(max_length=39, null=True))

    def backwards(self, orm):

        # Changing field 'PayPalNVP.ipaddress'
        db.alter_column('paypal_nvp', 'ipaddress', self.gf('django.db.models.fields.IPAddressField')(default='0.0.0.0', max_length=15))

    models = {
        user_model_label: {
            'Meta': {'object_name': User.__name__,
                     'db_table': "'%s'" % User._meta.db_table
            },
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
        },
        u'pro.paypalnvp': {
            'Meta': {'object_name': 'PayPalNVP', 'db_table': "'paypal_nvp'"},
            'ack': ('django.db.models.fields.CharField', [], {'max_length': '32', 'blank': 'True'}),
            'city': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'correlationid': ('django.db.models.fields.CharField', [], {'max_length': '32', 'blank': 'True'}),
            'countrycode': ('django.db.models.fields.CharField', [], {'max_length': '2', 'blank': 'True'}),
            'created_at': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'custom': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'firstname': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'flag': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'flag_code': ('django.db.models.fields.CharField', [], {'max_length': '32', 'blank': 'True'}),
            'flag_info': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'invnum': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'ipaddress': ('django.db.models.fields.GenericIPAddressField', [], {'max_length': '39', 'null': 'True', 'blank': 'True'}),
            'lastname': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'method': ('django.db.models.fields.CharField', [], {'max_length': '64', 'blank': 'True'}),
            'payerid': ('django.db.models.fields.CharField', [], {'max_length': '64', 'blank': 'True'}),
            'profileid': ('django.db.models.fields.CharField', [], {'max_length': '32', 'blank': 'True'}),
            'profilereference': ('django.db.models.fields.CharField', [], {'max_length': '128', 'blank': 'True'}),
            'profilestatus': ('django.db.models.fields.CharField', [], {'max_length': '32', 'blank': 'True'}),
            'query': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'response': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'state': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'street': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'timestamp': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'token': ('django.db.models.fields.CharField', [], {'max_length': '64', 'blank': 'True'}),
            'updated_at': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm[user_orm_label]", 'null': 'True', 'blank': 'True'}),
            'zip': ('django.db.models.fields.CharField', [], {'max_length': '32', 'blank': 'True'})
        }
    }

    complete_apps = ['pro']


from __future__ import unicode_literals

from django.test import TestCase
from django.test.utils import override_settings

from paypal.standard.forms import PayPalPaymentsForm
from paypal.standard.ipn.forms import PayPalIPNForm


class PaymentsFormTest(TestCase):

    def test_form_render(self):
        f = PayPalPaymentsForm(initial={'business': 'me@mybusiness.com',
                                        'amount': '10.50',
                                        'shipping': '2.00',
                                        'return_url': 'https://example.com/return_url',
                                        })
        rendered = f.render()
        self.assertIn('''action="https://www.sandbox.paypal.com/cgi-bin/webscr"''', rendered)
        self.assertIn('''value="me@mybusiness.com"''', rendered)
        self.assertIn('''value="2.00"''', rendered)
        self.assertIn('''value="10.50"''', rendered)
        self.assertIn('''buynowCC''', rendered)

    def test_form_endpont(self):
        with self.settings(PAYPAL_TEST=False):
            f = PayPalPaymentsForm(initial={})
            self.assertNotIn('sandbox', f.render())

    @override_settings(PAYPAL_RECEIVER_EMAIL='me@mybusiness.com')
    def test_form_render_deprecated_paypal_receiver_email(self):
        f = PayPalPaymentsForm(initial={'amount': '10.50',
                                        'shipping': '2.00',
                                        })
        rendered = f.render()
        self.assertIn('''action="https://www.sandbox.paypal.com/cgi-bin/webscr"''', rendered)
        self.assertIn('''value="me@mybusiness.com"''', rendered)
        self.assertIn('''value="2.00"''', rendered)
        self.assertIn('''value="10.50"''', rendered)
        self.assertIn('''buynowCC''', rendered)

    def test_invalid_date_format(self):
        data = {'payment_date': "2015-10-25 01:21:32"}
        form = PayPalIPNForm(data)
        self.assertFalse(form.is_valid())
        self.assertIn(
            form.errors,
            [
                {
                    'payment_date': ['Invalid date format '
                                     '2015-10-25 01:21:32: '
                                     'need more than 2 values to unpack']
                },
                {
                    'payment_date': ['Invalid date format '
                                     '2015-10-25 01:21:32: '
                                     'not enough values to unpack '
                                     '(expected 5, got 2)']
                }
            ]
        )



# -*- coding: utf-8 -*-

from south.db import db
from django.db import models
from paypal.standard.pdt.models import *


class Migration:
    def forwards(self, orm):
        # Adding model 'PayPalPDT'
        db.create_table('paypal_pdt', (
            ('id', models.AutoField(primary_key=True)),
            ('business', models.CharField(max_length=127, blank=True)),
            ('charset', models.CharField(max_length=32, blank=True)),
            ('custom', models.CharField(max_length=255, blank=True)),
            ('notify_version', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
            ('parent_txn_id', models.CharField("Parent Transaction ID", max_length=19, blank=True)),
            ('receiver_email', models.EmailField(max_length=127, blank=True)),
            ('receiver_id', models.CharField(max_length=127, blank=True)),
            ('residence_country', models.CharField(max_length=2, blank=True)),
            ('test_ipn', models.BooleanField(default=False, blank=True)),
            ('txn_id', models.CharField("Transaction ID", max_length=19, blank=True)),
            ('txn_type', models.CharField("Transaction Type", max_length=128, blank=True)),
            ('verify_sign', models.CharField(max_length=255, blank=True)),
            ('address_country', models.CharField(max_length=64, blank=True)),
            ('address_city', models.CharField(max_length=40, blank=True)),
            ('address_country_code', models.CharField(max_length=64, blank=True)),
            ('address_name', models.CharField(max_length=128, blank=True)),
            ('address_state', models.CharField(max_length=40, blank=True)),
            ('address_status', models.CharField(max_length=11, blank=True)),
            ('address_street', models.CharField(max_length=200, blank=True)),
            ('address_zip', models.CharField(max_length=20, blank=True)),
            ('contact_phone', models.CharField(max_length=20, blank=True)),
            ('first_name', models.CharField(max_length=64, blank=True)),
            ('last_name', models.CharField(max_length=64, blank=True)),
            ('payer_business_name', models.CharField(max_length=127, blank=True)),
            ('payer_email', models.CharField(max_length=127, blank=True)),
            ('payer_id', models.CharField(max_length=13, blank=True)),
            ('auth_amount', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
            ('auth_exp', models.CharField(max_length=28, blank=True)),
            ('auth_id', models.CharField(max_length=19, blank=True)),
            ('auth_status', models.CharField(max_length=9, blank=True)),
            ('exchange_rate', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=16, blank=True)),
            ('invoice', models.CharField(max_length=127, blank=True)),
            ('item_name', models.CharField(max_length=127, blank=True)),
            ('item_number', models.CharField(max_length=127, blank=True)),
            ('mc_currency', models.CharField(default='USD', max_length=32, blank=True)),
            ('mc_fee', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
            ('mc_gross', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
            ('mc_handling', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
            ('mc_shipping', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
            ('memo', models.CharField(max_length=255, blank=True)),
            ('num_cart_items', models.IntegerField(default=0, null=True, blank=True)),
            ('option_name1', models.CharField(max_length=64, blank=True)),
            ('option_name2', models.CharField(max_length=64, blank=True)),
            ('payer_status', models.CharField(max_length=10, blank=True)),
            ('payment_date', models.DateTimeField(null=True, blank=True)),
            ('payment_gross', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
            ('payment_status', models.CharField(max_length=9, blank=True)),
            ('payment_type', models.CharField(max_length=7, blank=True)),
            ('pending_reason', models.CharField(max_length=14, blank=True)),
            ('protection_eligibility', models.CharField(max_length=32, blank=True)),
            ('quantity', models.IntegerField(default=1, null=True, blank=True)),
            ('reason_code', models.CharField(max_length=15, blank=True)),
            (
            'remaining_settle', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
            ('settle_amount', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
            ('settle_currency', models.CharField(max_length=32, blank=True)),
            ('shipping', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
            ('shipping_method', models.CharField(max_length=255, blank=True)),
            ('tax', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
            ('transaction_entity', models.CharField(max_length=7, blank=True)),
            ('auction_buyer_id', models.CharField(max_length=64, blank=True)),
            ('auction_closing_date', models.DateTimeField(null=True, blank=True)),
            ('auction_multi_item', models.IntegerField(default=0, null=True, blank=True)),
            ('for_auction', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
            ('amount', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
            (
            'amount_per_cycle', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
            ('initial_payment_amount',
             models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
            ('next_payment_date', models.DateTimeField(null=True, blank=True)),
            ('outstanding_balance',
             models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
            ('payment_cycle', models.CharField(max_length=32, blank=True)),
            ('period_type', models.CharField(max_length=32, blank=True)),
            ('product_name', models.CharField(max_length=128, blank=True)),
            ('product_type', models.CharField(max_length=128, blank=True)),
            ('profile_status', models.CharField(max_length=32, blank=True)),
            ('recurring_payment_id', models.CharField(max_length=128, blank=True)),
            ('rp_invoice_id', models.CharField(max_length=127, blank=True)),
            ('time_created', models.DateTimeField(null=True, blank=True)),
            ('amount1', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
            ('amount2', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
            ('amount3', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
            ('mc_amount1', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
            ('mc_amount2', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
            ('mc_amount3', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
            ('password', models.CharField(max_length=24, blank=True)),
            ('period1', models.CharField(max_length=32, blank=True)),
            ('period2', models.CharField(max_length=32, blank=True)),
            ('period3', models.CharField(max_length=32, blank=True)),
            ('reattempt', models.CharField(max_length=1, blank=True)),
            ('recur_times', models.IntegerField(default=0, null=True, blank=True)),
            ('recurring', models.CharField(max_length=1, blank=True)),
            ('retry_at', models.DateTimeField(null=True, blank=True)),
            ('subscr_date', models.DateTimeField(null=True, blank=True)),
            ('subscr_effective', models.DateTimeField(null=True, blank=True)),
            ('subscr_id', models.CharField(max_length=19, blank=True)),
            ('username', models.CharField(max_length=64, blank=True)),
            ('case_creation_date', models.DateTimeField(null=True, blank=True)),
            ('case_id', models.CharField(max_length=14, blank=True)),
            ('case_type', models.CharField(max_length=24, blank=True)),
            ('receipt_id', models.CharField(max_length=64, blank=True)),
            ('currency_code', models.CharField(default='USD', max_length=32, blank=True)),
            ('handling_amount', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
            ('transaction_subject', models.CharField(max_length=255, blank=True)),
            ('ipaddress', models.IPAddressField(blank=True)),
            ('flag', models.BooleanField(default=False, blank=True)),
            ('flag_code', models.CharField(max_length=16, blank=True)),
            ('flag_info', models.TextField(blank=True)),
            ('query', models.TextField(blank=True)),
            ('response', models.TextField(blank=True)),
            ('created_at', models.DateTimeField(auto_now_add=True)),
            ('updated_at', models.DateTimeField(auto_now=True)),
            ('from_view', models.CharField(max_length=6, null=True, blank=True)),
            ('amt', models.DecimalField(default=0, null=True, max_digits=64, decimal_places=2, blank=True)),
            ('cm', models.CharField(max_length=255, blank=True)),
            ('sig', models.CharField(max_length=255, blank=True)),
            ('tx', models.CharField(max_length=255, blank=True)),
            ('st', models.CharField(max_length=32, blank=True)),
        ))
        db.send_create_signal('pdt', ['PayPalPDT'])


    def backwards(self, orm):
        # Deleting model 'PayPalPDT'
        db.delete_table('paypal_pdt')


    models = {
        'pdt.paypalpdt': {
            'Meta': {'db_table': '"paypal_pdt"'},
            'address_city': ('models.CharField', [], {'max_length': '40', 'blank': 'True'}),
            'address_country': ('models.CharField', [], {'max_length': '64', 'blank': 'True'}),
            'address_country_code': ('models.CharField', [], {'max_length': '64', 'blank': 'True'}),
            'address_name': ('models.CharField', [], {'max_length': '128', 'blank': 'True'}),
            'address_state': ('models.CharField', [], {'max_length': '40', 'blank': 'True'}),
            'address_status': ('models.CharField', [], {'max_length': '11', 'blank': 'True'}),
            'address_street': ('models.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'address_zip': ('models.CharField', [], {'max_length': '20', 'blank': 'True'}),
            'amount': ('models.DecimalField', [],
                       {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2', 'blank': 'True'}),
            'amount1': ('models.DecimalField', [],
                        {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2', 'blank': 'True'}),
            'amount2': ('models.DecimalField', [],
                        {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2', 'blank': 'True'}),
            'amount3': ('models.DecimalField', [],
                        {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2', 'blank': 'True'}),
            'amount_per_cycle': ('models.DecimalField', [],
                                 {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2',
                                  'blank': 'True'}),
            'amt': ('models.DecimalField', [],
                    {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2', 'blank': 'True'}),
            'auction_buyer_id': ('models.CharField', [], {'max_length': '64', 'blank': 'True'}),
            'auction_closing_date': ('models.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'auction_multi_item': ('models.IntegerField', [], {'default': '0', 'null': 'True', 'blank': 'True'}),
            'auth_amount': ('models.DecimalField', [],
                            {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2',
                             'blank': 'True'}),
            'auth_exp': ('models.CharField', [], {'max_length': '28', 'blank': 'True'}),
            'auth_id': ('models.CharField', [], {'max_length': '19', 'blank': 'True'}),
            'auth_status': ('models.CharField', [], {'max_length': '9', 'blank': 'True'}),
            'business': ('models.CharField', [], {'max_length': '127', 'blank': 'True'}),
            'case_creation_date': ('models.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'case_id': ('models.CharField', [], {'max_length': '14', 'blank': 'True'}),
            'case_type': ('models.CharField', [], {'max_length': '24', 'blank': 'True'}),
            'charset': ('models.CharField', [], {'max_length': '32', 'blank': 'True'}),
            'cm': ('models.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'contact_phone': ('models.CharField', [], {'max_length': '20', 'blank': 'True'}),
            'created_at': ('models.DateTimeField', [], {'auto_now_add': 'True'}),
            'currency_code': ('models.CharField', [], {'default': "'USD'", 'max_length': '32', 'blank': 'True'}),
            'custom': ('models.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'exchange_rate': ('models.DecimalField', [],
                              {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '16',
                               'blank': 'True'}),
            'first_name': ('models.CharField', [], {'max_length': '64', 'blank': 'True'}),
            'flag': ('models.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'flag_code': ('models.CharField', [], {'max_length': '16', 'blank': 'True'}),
            'flag_info': ('models.TextField', [], {'blank': 'True'}),
            'for_auction': ('models.DecimalField', [],
                            {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2',
                             'blank': 'True'}),
            'from_view': ('models.CharField', [], {'max_length': '6', 'null': 'True', 'blank': 'True'}),
            'handling_amount': ('models.DecimalField', [],
                                {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2',
                                 'blank': 'True'}),
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'initial_payment_amount': ('models.DecimalField', [],
                                       {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2',
                                        'blank': 'True'}),
            'invoice': ('models.CharField', [], {'max_length': '127', 'blank': 'True'}),
            'ipaddress': ('models.IPAddressField', [], {'blank': 'True'}),
            'item_name': ('models.CharField', [], {'max_length': '127', 'blank': 'True'}),
            'item_number': ('models.CharField', [], {'max_length': '127', 'blank': 'True'}),
            'last_name': ('models.CharField', [], {'max_length': '64', 'blank': 'True'}),
            'mc_amount1': ('models.DecimalField', [],
                           {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2',
                            'blank': 'True'}),
            'mc_amount2': ('models.DecimalField', [],
                           {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2',
                            'blank': 'True'}),
            'mc_amount3': ('models.DecimalField', [],
                           {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2',
                            'blank': 'True'}),
            'mc_currency': ('models.CharField', [], {'default': "'USD'", 'max_length': '32', 'blank': 'True'}),
            'mc_fee': ('models.DecimalField', [],
                       {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2', 'blank': 'True'}),
            'mc_gross': ('models.DecimalField', [],
                         {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2', 'blank': 'True'}),
            'mc_handling': ('models.DecimalField', [],
                            {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2',
                             'blank': 'True'}),
            'mc_shipping': ('models.DecimalField', [],
                            {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2',
                             'blank': 'True'}),
            'memo': ('models.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'next_payment_date': ('models.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'notify_version': ('models.DecimalField', [],
                               {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2',
                                'blank': 'True'}),
            'num_cart_items': ('models.IntegerField', [], {'default': '0', 'null': 'True', 'blank': 'True'}),
            'option_name1': ('models.CharField', [], {'max_length': '64', 'blank': 'True'}),
            'option_name2': ('models.CharField', [], {'max_length': '64', 'blank': 'True'}),
            'outstanding_balance': ('models.DecimalField', [],
                                    {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2',
                                     'blank': 'True'}),
            'parent_txn_id': ('models.CharField', ['"Parent Transaction ID"'], {'max_length': '19', 'blank': 'True'}),
            'password': ('models.CharField', [], {'max_length': '24', 'blank': 'True'}),
            'payer_business_name': ('models.CharField', [], {'max_length': '127', 'blank': 'True'}),
            'payer_email': ('models.CharField', [], {'max_length': '127', 'blank': 'True'}),
            'payer_id': ('models.CharField', [], {'max_length': '13', 'blank': 'True'}),
            'payer_status': ('models.CharField', [], {'max_length': '10', 'blank': 'True'}),
            'payment_cycle': ('models.CharField', [], {'max_length': '32', 'blank': 'True'}),
            'payment_date': ('models.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'payment_gross': ('models.DecimalField', [],
                              {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2',
                               'blank': 'True'}),
            'payment_status': ('models.CharField', [], {'max_length': '9', 'blank': 'True'}),
            'payment_type': ('models.CharField', [], {'max_length': '7', 'blank': 'True'}),
            'pending_reason': ('models.CharField', [], {'max_length': '14', 'blank': 'True'}),
            'period1': ('models.CharField', [], {'max_length': '32', 'blank': 'True'}),
            'period2': ('models.CharField', [], {'max_length': '32', 'blank': 'True'}),
            'period3': ('models.CharField', [], {'max_length': '32', 'blank': 'True'}),
            'period_type': ('models.CharField', [], {'max_length': '32', 'blank': 'True'}),
            'product_name': ('models.CharField', [], {'max_length': '128', 'blank': 'True'}),
            'product_type': ('models.CharField', [], {'max_length': '128', 'blank': 'True'}),
            'profile_status': ('models.CharField', [], {'max_length': '32', 'blank': 'True'}),
            'protection_eligibility': ('models.CharField', [], {'max_length': '32', 'blank': 'True'}),
            'quantity': ('models.IntegerField', [], {'default': '1', 'null': 'True', 'blank': 'True'}),
            'query': ('models.TextField', [], {'blank': 'True'}),
            'reason_code': ('models.CharField', [], {'max_length': '15', 'blank': 'True'}),
            'reattempt': ('models.CharField', [], {'max_length': '1', 'blank': 'True'}),
            'receipt_id': ('models.CharField', [], {'max_length': '64', 'blank': 'True'}),
            'receiver_email': ('models.EmailField', [], {'max_length': '127', 'blank': 'True'}),
            'receiver_id': ('models.CharField', [], {'max_length': '127', 'blank': 'True'}),
            'recur_times': ('models.IntegerField', [], {'default': '0', 'null': 'True', 'blank': 'True'}),
            'recurring': ('models.CharField', [], {'max_length': '1', 'blank': 'True'}),
            'recurring_payment_id': ('models.CharField', [], {'max_length': '128', 'blank': 'True'}),
            'remaining_settle': ('models.DecimalField', [],
                                 {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2',
                                  'blank': 'True'}),
            'residence_country': ('models.CharField', [], {'max_length': '2', 'blank': 'True'}),
            'response': ('models.TextField', [], {'blank': 'True'}),
            'retry_at': ('models.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'rp_invoice_id': ('models.CharField', [], {'max_length': '127', 'blank': 'True'}),
            'settle_amount': ('models.DecimalField', [],
                              {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2',
                               'blank': 'True'}),
            'settle_currency': ('models.CharField', [], {'max_length': '32', 'blank': 'True'}),
            'shipping': ('models.DecimalField', [],
                         {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2', 'blank': 'True'}),
            'shipping_method': ('models.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'sig': ('models.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'st': ('models.CharField', [], {'max_length': '32', 'blank': 'True'}),
            'subscr_date': ('models.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'subscr_effective': ('models.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'subscr_id': ('models.CharField', [], {'max_length': '19', 'blank': 'True'}),
            'tax': ('models.DecimalField', [],
                    {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2', 'blank': 'True'}),
            'test_ipn': ('models.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'time_created': ('models.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'transaction_entity': ('models.CharField', [], {'max_length': '7', 'blank': 'True'}),
            'transaction_subject': ('models.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'tx': ('models.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'txn_id': ('models.CharField', ['"Transaction ID"'], {'max_length': '19', 'blank': 'True'}),
            'txn_type': ('models.CharField', ['"Transaction Type"'], {'max_length': '128', 'blank': 'True'}),
            'updated_at': ('models.DateTimeField', [], {'auto_now': 'True'}),
            'username': ('models.CharField', [], {'max_length': '64', 'blank': 'True'}),
            'verify_sign': ('models.CharField', [], {'max_length': '255', 'blank': 'True'})
        }
    }

    complete_apps = ['pdt']



import sys
import warnings

from django.conf import settings


class PayPalSettingsError(Exception):
    """Raised when settings be bad."""


# API Endpoints.
POSTBACK_ENDPOINT = "https://www.paypal.com/cgi-bin/webscr"
SANDBOX_POSTBACK_ENDPOINT = "https://www.sandbox.paypal.com/cgi-bin/webscr"

# Images
BUY_BUTTON_IMAGE = getattr(settings, "PAYPAL_BUY_BUTTON_IMAGE",
                           "https://www.paypal.com/en_US/i/btn/btn_buynowCC_LG.gif")
SUBSCRIPTION_BUTTON_IMAGE = getattr(settings, "PAYPAL_SUBSCRIPTION_BUTTON_IMAGE",
                             "https://www.paypal.com/en_US/i/btn/btn_subscribeCC_LG.gif")
DONATION_BUTTON_IMAGE = getattr(settings, "PAYPAL_DONATION_BUTTON_IMAGE",
                                "https://www.paypal.com/en_US/i/btn/btn_donateCC_LG.gif")


deprecated_settings = [
    ('IMAGE', 'BUY_BUTTON_IMAGE'),
    ('SUBSCRIPTION_IMAGE', 'SANDBOX_SUBSCRIPTION_BUTTON_IMAGE'),
    ('DONATION_IMAGE', 'DONATION_BUTTON_IMAGE'),
]

removed_settings = [
    'SANDBOX_IMAGE',
    'SUBSCRIPTION_SANDBOX_IMAGE',
    'DONATION_SANDBOX_IMAGE',
]

for old, new in deprecated_settings:
    old_setting = 'PAYPAL_' + old
    new_setting = 'PAYPAL_' + new
    old_setting_val = getattr(settings, old_setting, None)
    if old_setting_val is not None:
        warnings.warn(
            "Setting {0} is deprecated - use {1} instead.".format(
                old_setting, new_setting),
            DeprecationWarning)
        if hasattr(settings, new_setting):
            warnings.warn(
                "You have both old setting {0} and new setting {1} in your settings, "
                "please remove the old setting.".format(
                    old_setting, new_setting),
                DeprecationWarning)
        else:
            # use the value from the deprecated setting
            setattr(sys.modules[__name__], new, old_setting_val)

for old in removed_settings:
    old_setting = 'PAYPAL_' + old
    old_setting_val = getattr(settings, old_setting, None)
    if old_setting_val is not None:
        warnings.warn(
            "Setting {0} has been removed and is ignored.".format(old_setting),
            DeprecationWarning)


# Paypal Encrypt Certificate
PAYPAL_PRIVATE_CERT = getattr(settings, 'PAYPAL_PRIVATE_CERT', None)
PAYPAL_PUBLIC_CERT = getattr(settings, 'PAYPAL_PUBLIC_CERT', None)
PAYPAL_CERT = getattr(settings, 'PAYPAL_CERT', None)
PAYPAL_CERT_ID = getattr(settings, 'PAYPAL_CERT_ID', None)



import warnings

from django.dispatch import Signal


class DeprecatedSignal(Signal):

    def __init__(self, deprecation_message, *args, **kwargs):
        self.deprecation_message = deprecation_message
        super(DeprecatedSignal, self).__init__(*args, **kwargs)

    def connect(self, *args, **kwargs):
        warnings.warn(
            self.deprecation_message, DeprecationWarning, stacklevel=2)
        return super(DeprecatedSignal, self).connect(*args, **kwargs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import logging
from datetime import datetime
from warnings import warn

from django import forms
from django.conf import settings
from django.core.exceptions import ValidationError
from django.utils import timezone
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _

from paypal.standard.conf import (
    BUY_BUTTON_IMAGE, DONATION_BUTTON_IMAGE, PAYPAL_CERT, PAYPAL_CERT_ID, PAYPAL_PRIVATE_CERT, PAYPAL_PUBLIC_CERT,
    POSTBACK_ENDPOINT, SANDBOX_POSTBACK_ENDPOINT, SUBSCRIPTION_BUTTON_IMAGE
)
from paypal.standard.widgets import ReservedValueHiddenInput, ValueHiddenInput
from paypal.utils import warn_untested

log = logging.getLogger(__name__)


# PayPal date format e.g.:
#   20:18:05 Jan 30, 2009 PST
#
# PayPal dates have been spotted in the wild with these formats, beware!
#
# %H:%M:%S %b. %d, %Y PST
# %H:%M:%S %b. %d, %Y PDT
# %H:%M:%S %b %d, %Y PST
# %H:%M:%S %b %d, %Y PDT
#
# To avoid problems with different locales, we don't rely on datetime.strptime,
# which is locale dependent, but do custom parsing in PayPalDateTimeField

MONTHS = [
    'Jan', 'Feb', 'Mar', 'Apr',
    'May', 'Jun', 'Jul', 'Aug',
    'Sep', 'Oct', 'Nov', 'Dec',
]


class PayPalDateTimeField(forms.DateTimeField):

    def to_python(self, value):
        if value in self.empty_values:
            return None

        if isinstance(value, datetime):
            return value

        value = value.strip()

        try:
            time_part, month_part, day_part, year_part, zone_part = value.split()
            month_part = month_part.strip(".")
            day_part = day_part.strip(",")
            month = MONTHS.index(month_part) + 1
            day = int(day_part)
            year = int(year_part)
            hour, minute, second = map(int, time_part.split(":"))
            dt = datetime(year, month, day, hour, minute, second)
        except ValueError as e:
            raise ValidationError(
                _("Invalid date format %(value)s: %(e)s"),
                params={'value': value, 'e': e},
                code="invalid_date"
            )

        if zone_part in ["PDT", "PST"]:
            # PST/PDT is 'US/Pacific'
            dt = timezone.pytz.timezone('US/Pacific').localize(
                dt, is_dst=zone_part == 'PDT')
            if not settings.USE_TZ:
                dt = timezone.make_naive(dt, timezone=timezone.utc)
        return dt


class PayPalPaymentsForm(forms.Form):
    """
    Creates a PayPal Payments Standard "Buy It Now" button, configured for a
    selling a single item with no shipping.

    For a full overview of all the fields you can set (there is a lot!) see:
    http://tinyurl.com/pps-integration

    Usage:
    >>> f = PayPalPaymentsForm(initial={'item_name':'Widget 001', ...})
    >>> f.render()
    u'<form action="https://www.paypal.com/cgi-bin/webscr" method="post"> ...'

    """
    CMD_CHOICES = (
        ("_xclick", "Buy now or Donations"),
        ("_donations", "Donations"),
        ("_cart", "Shopping cart"),
        ("_xclick-subscriptions", "Subscribe")
    )
    SHIPPING_CHOICES = ((1, "No shipping"), (0, "Shipping"))
    NO_NOTE_CHOICES = ((1, "No Note"), (0, "Include Note"))
    RECURRING_PAYMENT_CHOICES = (
        (1, "Subscription Payments Recur"),
        (0, "Subscription payments do not recur")
    )
    REATTEMPT_ON_FAIL_CHOICES = (
        (1, "reattempt billing on Failure"),
        (0, "Do Not reattempt on failure")
    )

    BUY = 'buy'
    SUBSCRIBE = 'subscribe'
    DONATE = 'donate'

    # Where the money goes.
    business = forms.CharField(widget=ValueHiddenInput())

    # Item information.
    amount = forms.IntegerField(widget=ValueHiddenInput())
    item_name = forms.CharField(widget=ValueHiddenInput())
    item_number = forms.CharField(widget=ValueHiddenInput())
    quantity = forms.CharField(widget=ValueHiddenInput())

    # Subscription Related.
    a1 = forms.CharField(widget=ValueHiddenInput())   # Trial 1 Price
    p1 = forms.CharField(widget=ValueHiddenInput())   # Trial 1 Duration
    t1 = forms.CharField(widget=ValueHiddenInput())   # Trial 1 unit of Duration, default to Month
    a2 = forms.CharField(widget=ValueHiddenInput())   # Trial 2 Price
    p2 = forms.CharField(widget=ValueHiddenInput())   # Trial 2 Duration
    t2 = forms.CharField(widget=ValueHiddenInput())   # Trial 2 unit of Duration, default to Month
    a3 = forms.CharField(widget=ValueHiddenInput())   # Subscription Price
    p3 = forms.CharField(widget=ValueHiddenInput())   # Subscription Duration
    t3 = forms.CharField(widget=ValueHiddenInput())   # Subscription unit of Duration, default to Month
    src = forms.CharField(widget=ValueHiddenInput())  # Is billing recurring? default to yes
    sra = forms.CharField(widget=ValueHiddenInput())  # Reattempt billing on failed cc transaction
    no_note = forms.CharField(widget=ValueHiddenInput())
    # Can be either 1 or 2. 1 = modify or allow new subscription creation, 2 = modify only
    modify = forms.IntegerField(widget=ValueHiddenInput())  # Are we modifying an existing subscription?

    # Localization / PayPal Setup
    lc = forms.CharField(widget=ValueHiddenInput())
    page_style = forms.CharField(widget=ValueHiddenInput())
    cbt = forms.CharField(widget=ValueHiddenInput())

    # IPN control.
    notify_url = forms.CharField(widget=ValueHiddenInput())
    cancel_return = forms.CharField(widget=ValueHiddenInput())
    return_url = forms.CharField(widget=ReservedValueHiddenInput(attrs={"name": "return"}))
    custom = forms.CharField(widget=ValueHiddenInput())
    invoice = forms.CharField(widget=ValueHiddenInput())

    # Default fields.
    cmd = forms.ChoiceField(widget=forms.HiddenInput(), initial=CMD_CHOICES[0][0])
    charset = forms.CharField(widget=forms.HiddenInput(), initial="utf-8")
    currency_code = forms.CharField(widget=forms.HiddenInput(), initial="USD")
    no_shipping = forms.ChoiceField(widget=forms.HiddenInput(), choices=SHIPPING_CHOICES,
                                    initial=SHIPPING_CHOICES[0][0])

    def __init__(self, button_type="buy", *args, **kwargs):
        super(PayPalPaymentsForm, self).__init__(*args, **kwargs)
        self.button_type = button_type
        if 'initial' in kwargs:
            kwargs['initial'] = self._fix_deprecated_paypal_receiver_email(kwargs['initial'])
            # Dynamically create, so we can support everything PayPal does.
            for k, v in kwargs['initial'].items():
                if k not in self.base_fields:
                    self.fields[k] = forms.CharField(label=k, widget=ValueHiddenInput(), initial=v)

    def _fix_deprecated_paypal_receiver_email(self, initial_args):
        if 'business' not in initial_args:
            if hasattr(settings, 'PAYPAL_RECEIVER_EMAIL'):
                warn("""The use of the settings.PAYPAL_RECEIVER_EMAIL is Deprecated.
                        The keyword business argument must be given to PayPalPaymentsForm
                        on creation""", DeprecationWarning)
                initial_args['business'] = settings.PAYPAL_RECEIVER_EMAIL
        return initial_args

    def test_mode(self):
        return getattr(settings, 'PAYPAL_TEST', True)

    def get_endpoint(self):
        "Returns the endpoint url for the form."
        if self.test_mode():
            return SANDBOX_POSTBACK_ENDPOINT
        else:
            return POSTBACK_ENDPOINT

    def render(self):
        return format_html(u"""<form action="{0}" method="post">
    {1}
    <input type="image" src="{2}" border="0" name="submit" alt="Buy it Now" />
</form>""", self.get_endpoint(), self.as_p(), self.get_image())

    def sandbox(self):
        "Deprecated.  Use self.render() instead."
        import warnings
        warnings.warn("""PaypalPaymentsForm.sandbox() is deprecated.
                    Use the render() method instead.""", DeprecationWarning)
        return self.render()

    def get_image(self):
        return {
            self.SUBSCRIBE: SUBSCRIPTION_BUTTON_IMAGE,
            self.BUY: BUY_BUTTON_IMAGE,
            self.DONATE: DONATION_BUTTON_IMAGE,
        }[self.button_type]

    def is_transaction(self):
        warn_untested()
        return not self.is_subscription()

    def is_donation(self):
        warn_untested()
        return self.button_type == self.DONATE

    def is_subscription(self):
        warn_untested()
        return self.button_type == self.SUBSCRIBE


class PayPalEncryptedPaymentsForm(PayPalPaymentsForm):
    """
    Creates a PayPal Encrypted Payments "Buy It Now" button.
    Requires the M2Crypto package.

    Based on example at:
    http://blog.mauveweb.co.uk/2007/10/10/paypal-with-django/

    """

    def __init__(self, private_cert=PAYPAL_PRIVATE_CERT, public_cert=PAYPAL_PUBLIC_CERT,
            paypal_cert=PAYPAL_CERT, cert_id=PAYPAL_CERT_ID, *args, **kwargs):
        warn_untested()
        super(PayPalEncryptedPaymentsForm, self).__init__(*args, **kwargs)
        self.private_cert = private_cert
        self.public_cert = public_cert
        self.paypal_cert = paypal_cert
        self.cert_id = cert_id

    def _encrypt(self):
        """Use your key thing to encrypt things."""
        warn_untested()
        from M2Crypto import BIO, SMIME, X509

        # Iterate through the fields and pull out the ones that have a value.
        plaintext = 'cert_id=%s\n' % self.cert_id
        for name, field in self.fields.items():
            value = None
            if name in self.initial:
                value = self.initial[name]
            elif field.initial is not None:
                value = field.initial
            if value is not None:
                # @@@ Make this less hackish and put it in the widget.
                if name == "return_url":
                    name = "return"
                plaintext += u'%s=%s\n' % (name, value)
        plaintext = plaintext.encode('utf-8')

        # Begin crypto weirdness.
        s = SMIME.SMIME()
        s.load_key_bio(BIO.openfile(self.private_cert), BIO.openfile(self.public_cert))
        p7 = s.sign(BIO.MemoryBuffer(plaintext), flags=SMIME.PKCS7_BINARY)
        x509 = X509.load_cert_bio(BIO.openfile(self.paypal_cert))
        sk = X509.X509_Stack()
        sk.push(x509)
        s.set_x509_stack(sk)
        s.set_cipher(SMIME.Cipher('des_ede3_cbc'))
        tmp = BIO.MemoryBuffer()
        p7.write_der(tmp)
        p7 = s.encrypt(tmp, flags=SMIME.PKCS7_BINARY)
        out = BIO.MemoryBuffer()
        p7.write(out)
        return out.read()

    def as_p(self):
        warn_untested()
        return mark_safe(u"""
<input type="hidden" name="cmd" value="_s-xclick" />
<input type="hidden" name="encrypted" value="%s" />
        """ % self._encrypt())


class PayPalSharedSecretEncryptedPaymentsForm(PayPalEncryptedPaymentsForm):
    """
    Creates a PayPal Encrypted Payments "Buy It Now" button with a Shared Secret.
    Shared secrets should only be used when your IPN endpoint is on HTTPS.

    Adds a secret to the notify_url based on the contents of the form.

    """

    def __init__(self, *args, **kwargs):
        "Make the secret from the form initial data and slip it into the form."
        warn_untested()
        from paypal.standard.helpers import make_secret

        super(PayPalSharedSecretEncryptedPaymentsForm, self).__init__(*args, **kwargs)
        # @@@ Attach the secret parameter in a way that is safe for other query params.
        secret_param = "?secret=%s" % make_secret(self)
        # Initial data used in form construction overrides defaults
        if 'notify_url' in self.initial:
            self.initial['notify_url'] += secret_param
        else:
            self.fields['notify_url'].initial += secret_param


class PayPalStandardBaseForm(forms.ModelForm):
    """Form used to receive and record PayPal IPN/PDT."""
    # PayPal dates have non-standard formats.
    time_created = PayPalDateTimeField(required=False)
    payment_date = PayPalDateTimeField(required=False)
    next_payment_date = PayPalDateTimeField(required=False)
    subscr_date = PayPalDateTimeField(required=False)
    subscr_effective = PayPalDateTimeField(required=False)
    retry_at = PayPalDateTimeField(required=False)
    case_creation_date = PayPalDateTimeField(required=False)
    auction_closing_date = PayPalDateTimeField(required=False)



"""
Note that sometimes you will get duplicate signals emitted, depending on configuration of your systems.
If you do encounter this, you will need to add the "dispatch_uid" to your connect handlers:
http://code.djangoproject.com/wiki/Signals#Helppost_saveseemstobeemittedtwiceforeachsave

"""
from __future__ import unicode_literals

from django.dispatch import Signal

from paypal.standard.signals import DeprecatedSignal

# Sent when a validated, non-duplicated IPN is received.
valid_ipn_received = Signal()

# Sent when a flagged IPN (e.g. duplicate, invalid) is received.
invalid_ipn_received = Signal()

# Deprecated signals:

# Sent when a payment is successfully processed.
payment_was_successful = DeprecatedSignal("payment_was_successful is deprecated, "
                                          "please migrate to valid_ipn_received instead")

# Sent when a payment is flagged.
payment_was_flagged = DeprecatedSignal("payment_was_flagged is deprecated, "
                                       "please migrate to invalid_ipn_received instead")

# Sent when a payment was refunded by the seller.
payment_was_refunded = DeprecatedSignal("payment_was_refunded is deprecated, "
                                        "please migrate to valid_ipn_received instead")

# Sent when a payment was reversed by the buyer.
payment_was_reversed = DeprecatedSignal("payment_was_reversed is deprecated, "
                                        "please migrate to valid_ipn_received instead")

# Sent when a subscription was cancelled.
subscription_cancel = DeprecatedSignal("subscription_cancel is deprecated, "
                                       "please migrate to valid_ipn_received instead")

# Sent when a subscription expires.
subscription_eot = DeprecatedSignal("subscription_eot is deprecated, "
                                    "please migrate to valid_ipn_received instead")

# Sent when a subscription was modified.
subscription_modify = DeprecatedSignal("subscription_modify is deprecated, "
                                       "please migrate to valid_ipn_received instead")

# Sent when a subscription is created.
subscription_signup = DeprecatedSignal("subscription_signup is deprecated, "
                                       "please migrate to valid_ipn_received instead")

# recurring_payment_profile_created
recurring_create = DeprecatedSignal("recurring_create is deprecated, "
                                    "please migrate to valid_ipn_received instead")

# recurring_payment
recurring_payment = DeprecatedSignal("recurring_payment is deprecated, "
                                     "please migrate to valid_ipn_received instead")

recurring_cancel = DeprecatedSignal("recurring_cancel is deprecated, "
                                    "please migrate to valid_ipn_received instead")

recurring_skipped = DeprecatedSignal("recurring_skipped is deprecated, "
                                     "please migrate to valid_ipn_received instead")

recurring_failed = DeprecatedSignal("recurring_failed is deprecated, "
                                    "please migrate to valid_ipn_received instead")



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):

        # Changing field 'PayPalIPN.custom'
        db.alter_column(u'paypal_ipn', 'custom', self.gf('django.db.models.fields.CharField')(max_length=256))

        # Changing field 'PayPalIPN.transaction_subject'
        db.alter_column(u'paypal_ipn', 'transaction_subject', self.gf('django.db.models.fields.CharField')(max_length=256))

    def backwards(self, orm):

        # Changing field 'PayPalIPN.custom'
        db.alter_column(u'paypal_ipn', 'custom', self.gf('django.db.models.fields.CharField')(max_length=255))

        # Changing field 'PayPalIPN.transaction_subject'
        db.alter_column(u'paypal_ipn', 'transaction_subject', self.gf('django.db.models.fields.CharField')(max_length=255))

    models = {
        u'ipn.paypalipn': {
            'Meta': {'object_name': 'PayPalIPN', 'db_table': "u'paypal_ipn'"},
            'address_city': ('django.db.models.fields.CharField', [], {'max_length': '40', 'blank': 'True'}),
            'address_country': ('django.db.models.fields.CharField', [], {'max_length': '64', 'blank': 'True'}),
            'address_country_code': ('django.db.models.fields.CharField', [], {'max_length': '64', 'blank': 'True'}),
            'address_name': ('django.db.models.fields.CharField', [], {'max_length': '128', 'blank': 'True'}),
            'address_state': ('django.db.models.fields.CharField', [], {'max_length': '40', 'blank': 'True'}),
            'address_status': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'address_street': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'address_zip': ('django.db.models.fields.CharField', [], {'max_length': '20', 'blank': 'True'}),
            'amount': ('django.db.models.fields.DecimalField', [], {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2', 'blank': 'True'}),
            'amount1': ('django.db.models.fields.DecimalField', [], {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2', 'blank': 'True'}),
            'amount2': ('django.db.models.fields.DecimalField', [], {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2', 'blank': 'True'}),
            'amount3': ('django.db.models.fields.DecimalField', [], {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2', 'blank': 'True'}),
            'amount_per_cycle': ('django.db.models.fields.DecimalField', [], {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2', 'blank': 'True'}),
            'auction_buyer_id': ('django.db.models.fields.CharField', [], {'max_length': '64', 'blank': 'True'}),
            'auction_closing_date': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'auction_multi_item': ('django.db.models.fields.IntegerField', [], {'default': '0', 'null': 'True', 'blank': 'True'}),
            'auth_amount': ('django.db.models.fields.DecimalField', [], {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2', 'blank': 'True'}),
            'auth_exp': ('django.db.models.fields.CharField', [], {'max_length': '28', 'blank': 'True'}),
            'auth_id': ('django.db.models.fields.CharField', [], {'max_length': '19', 'blank': 'True'}),
            'auth_status': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'business': ('django.db.models.fields.CharField', [], {'max_length': '127', 'blank': 'True'}),
            'case_creation_date': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'case_id': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'case_type': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'charset': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'contact_phone': ('django.db.models.fields.CharField', [], {'max_length': '20', 'blank': 'True'}),
            'created_at': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'currency_code': ('django.db.models.fields.CharField', [], {'default': "'USD'", 'max_length': '32', 'blank': 'True'}),
            'custom': ('django.db.models.fields.CharField', [], {'max_length': '256', 'blank': 'True'}),
            'exchange_rate': ('django.db.models.fields.DecimalField', [], {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '16', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '64', 'blank': 'True'}),
            'flag': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'flag_code': ('django.db.models.fields.CharField', [], {'max_length': '16', 'blank': 'True'}),
            'flag_info': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'for_auction': ('django.db.models.fields.DecimalField', [], {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2', 'blank': 'True'}),
            'from_view': ('django.db.models.fields.CharField', [], {'max_length': '6', 'null': 'True', 'blank': 'True'}),
            'handling_amount': ('django.db.models.fields.DecimalField', [], {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'initial_payment_amount': ('django.db.models.fields.DecimalField', [], {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2', 'blank': 'True'}),
            'invoice': ('django.db.models.fields.CharField', [], {'max_length': '127', 'blank': 'True'}),
            'ipaddress': ('django.db.models.fields.GenericIPAddressField', [], {'max_length': '39', 'null': 'True', 'blank': 'True'}),
            'item_name': ('django.db.models.fields.CharField', [], {'max_length': '127', 'blank': 'True'}),
            'item_number': ('django.db.models.fields.CharField', [], {'max_length': '127', 'blank': 'True'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '64', 'blank': 'True'}),
            'mc_amount1': ('django.db.models.fields.DecimalField', [], {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2', 'blank': 'True'}),
            'mc_amount2': ('django.db.models.fields.DecimalField', [], {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2', 'blank': 'True'}),
            'mc_amount3': ('django.db.models.fields.DecimalField', [], {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2', 'blank': 'True'}),
            'mc_currency': ('django.db.models.fields.CharField', [], {'default': "'USD'", 'max_length': '32', 'blank': 'True'}),
            'mc_fee': ('django.db.models.fields.DecimalField', [], {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2', 'blank': 'True'}),
            'mc_gross': ('django.db.models.fields.DecimalField', [], {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2', 'blank': 'True'}),
            'mc_handling': ('django.db.models.fields.DecimalField', [], {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2', 'blank': 'True'}),
            'mc_shipping': ('django.db.models.fields.DecimalField', [], {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2', 'blank': 'True'}),
            'memo': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'mp_id': ('django.db.models.fields.CharField', [], {'max_length': '128', 'null': 'True', 'blank': 'True'}),
            'next_payment_date': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'notify_version': ('django.db.models.fields.DecimalField', [], {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2', 'blank': 'True'}),
            'num_cart_items': ('django.db.models.fields.IntegerField', [], {'default': '0', 'null': 'True', 'blank': 'True'}),
            'option_name1': ('django.db.models.fields.CharField', [], {'max_length': '64', 'blank': 'True'}),
            'option_name2': ('django.db.models.fields.CharField', [], {'max_length': '64', 'blank': 'True'}),
            'outstanding_balance': ('django.db.models.fields.DecimalField', [], {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2', 'blank': 'True'}),
            'parent_txn_id': ('django.db.models.fields.CharField', [], {'max_length': '19', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '24', 'blank': 'True'}),
            'payer_business_name': ('django.db.models.fields.CharField', [], {'max_length': '127', 'blank': 'True'}),
            'payer_email': ('django.db.models.fields.CharField', [], {'max_length': '127', 'blank': 'True'}),
            'payer_id': ('django.db.models.fields.CharField', [], {'max_length': '13', 'blank': 'True'}),
            'payer_status': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'payment_cycle': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'payment_date': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'payment_gross': ('django.db.models.fields.DecimalField', [], {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2', 'blank': 'True'}),
            'payment_status': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'payment_type': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'pending_reason': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'period1': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'period2': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'period3': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'period_type': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'product_name': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'product_type': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'profile_status': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'protection_eligibility': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'quantity': ('django.db.models.fields.IntegerField', [], {'default': '1', 'null': 'True', 'blank': 'True'}),
            'query': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'reason_code': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'reattempt': ('django.db.models.fields.CharField', [], {'max_length': '1', 'blank': 'True'}),
            'receipt_id': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'receiver_email': ('django.db.models.fields.EmailField', [], {'max_length': '254', 'blank': 'True'}),
            'receiver_id': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'recur_times': ('django.db.models.fields.IntegerField', [], {'default': '0', 'null': 'True', 'blank': 'True'}),
            'recurring': ('django.db.models.fields.CharField', [], {'max_length': '1', 'blank': 'True'}),
            'recurring_payment_id': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'remaining_settle': ('django.db.models.fields.DecimalField', [], {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2', 'blank': 'True'}),
            'residence_country': ('django.db.models.fields.CharField', [], {'max_length': '2', 'blank': 'True'}),
            'response': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'retry_at': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'rp_invoice_id': ('django.db.models.fields.CharField', [], {'max_length': '127', 'blank': 'True'}),
            'settle_amount': ('django.db.models.fields.DecimalField', [], {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2', 'blank': 'True'}),
            'settle_currency': ('django.db.models.fields.CharField', [], {'max_length': '32', 'blank': 'True'}),
            'shipping': ('django.db.models.fields.DecimalField', [], {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2', 'blank': 'True'}),
            'shipping_method': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'subscr_date': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'subscr_effective': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'subscr_id': ('django.db.models.fields.CharField', [], {'max_length': '19', 'blank': 'True'}),
            'tax': ('django.db.models.fields.DecimalField', [], {'default': '0', 'null': 'True', 'max_digits': '64', 'decimal_places': '2', 'blank': 'True'}),
            'test_ipn': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'time_created': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'transaction_entity': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'transaction_subject': ('django.db.models.fields.CharField', [], {'max_length': '256', 'blank': 'True'}),
            'txn_id': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '255', 'blank': 'True'}),
            'txn_type': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'updated_at': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'max_length': '64', 'blank': 'True'}),
            'verify_sign': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'})
        }
    }

    complete_apps = ['ipn']


#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import requests

from paypal.standard.ipn.signals import (
    invalid_ipn_received, payment_was_flagged, payment_was_refunded, payment_was_reversed, payment_was_successful,
    recurring_cancel, recurring_create, recurring_failed, recurring_payment, recurring_skipped, subscription_cancel,
    subscription_eot, subscription_modify, subscription_signup, valid_ipn_received
)
from paypal.standard.models import PayPalStandardBase
from paypal.utils import warn_untested


class PayPalIPN(PayPalStandardBase):
    """Logs PayPal IPN interactions."""
    format = u"<IPN: %s %s>"

    class Meta:
        db_table = "paypal_ipn"
        verbose_name = "PayPal IPN"

    def _postback(self):
        """Perform PayPal Postback validation."""
        return requests.post(self.get_endpoint(), data=b"cmd=_notify-validate&" + self.query.encode("ascii")).content

    def _verify_postback(self):
        if self.response != "VERIFIED":
            self.set_flag("Invalid postback. ({0})".format(self.response))

    def send_signals(self):
        """Shout for the world to hear whether a txn was successful."""
        if self.flag:
            invalid_ipn_received.send(sender=self)
            payment_was_flagged.send(sender=self)
            return
        else:
            valid_ipn_received.send(sender=self)

        # Transaction signals:
        if self.is_transaction():
            if self.is_refund():
                payment_was_refunded.send(sender=self)
            elif self.is_reversed():
                payment_was_reversed.send(sender=self)
            else:
                payment_was_successful.send(sender=self)
        # Recurring payment signals:
        # XXX: Should these be merged with subscriptions?
        elif self.is_recurring():
            if self.is_recurring_create():
                recurring_create.send(sender=self)
            elif self.is_recurring_payment():
                warn_untested()
                recurring_payment.send(sender=self)
            elif self.is_recurring_cancel():
                recurring_cancel.send(sender=self)
            elif self.is_recurring_skipped():
                recurring_skipped.send(sender=self)
            elif self.is_recurring_failed():
                recurring_failed.send(sender=self)
        # Subscription signals:
        else:
            warn_untested()
            if self.is_subscription_cancellation():
                subscription_cancel.send(sender=self)
            elif self.is_subscription_signup():
                subscription_signup.send(sender=self)
            elif self.is_subscription_end_of_term():
                subscription_eot.send(sender=self)
            elif self.is_subscription_modified():
                subscription_modify.send(sender=self)

    def __repr__(self):
        return '<PayPalIPN id:{0}>'.format(self.id)

    def __str__(self):
        return "PayPalIPN: {0}".format(self.id)
